@@ -149,6 +149,7 @@ def launch(
 ) -> Iterator[Program]:
     command = [
         venv.spec.python_path().absolute(),
+        "-u",
         "-m",
         module,
     ] + args
@@ -204,25 +205,62 @@ def maybe_new_requirements_digest(venv: Venv) -> bytes | None:
 def ensure_digest_installed(venv: Venv, target_digest: bytes) -> None:
     if target_digest == venv.state.installed_digest:
         return venv
-    log.info("Uninstalling old requirements...")
+
+    time_since_last_attempt = time.time() - venv.state.when_last_update_attempt
+    if time_since_last_attempt < _MIN_TIME_BETWEEN_ATTEMPTS:
+        time.sleep(_MIN_TIME_BETWEEN_ATTEMPTS - time_since_last_attempt)
+    venv.state.when_last_update_attempt = time.time()
+
+    log.info("Calculating requirements...")
+
+    def no_comments(line: str) -> str:
+        return line.split("#", maxsplit=1)[0].strip()
+
+    def clean_requirement(line: str) -> str:
+        no_extras, _ = no_comments(line).split(";", maxsplit=1)
+        return no_extras.strip()
+
+    with open(venv.spec.requirements_file, "r") as f:
+        target_requirements = [
+            (clean_requirement(line), no_comments(line))
+            for line in f.readlines()
+            if line.strip()
+        ]
+        clean_target_requirements = [r[0] for r in target_requirements]
+
     pip_freeze = subprocess.run(
         [venv.spec.pip_path().absolute(), "freeze"],
         check=True,
         capture_output=True,
+        text=True,
     )
-    with open(venv.spec.requirements_file, "rb") as f:
-        requirements_to_install = [line for line in f.read().split(b"\n") if line]
-    if pip_freeze.stdout:
-        installed_requirements = [
-            line for line in pip_freeze.stdout.split(b"\n") if line
-        ]
-        requirements_to_uninstall = [
-            l for l in installed_requirements if l not in requirements_to_install
-        ]
+
+    installed_requirements = [
+        line.strip() for line in pip_freeze.stdout.split("\n") if line.strip()
+    ]
+
+    requirements_to_remove = [
+        line for line in installed_requirements if line not in clean_target_requirements
+    ]
+
+    requirements_to_install = [
+        line
+        for clean_requirement, line in target_requirements
+        if clean_requirement not in installed_requirements
+    ]
+
+    log.info(
+        "Remove:\n%s\n\nInstall:\n%s",
+        "\n".join(requirements_to_remove),
+        "\n".join(r[0] for r in requirements_to_install),
+    )
+
+    if requirements_to_remove:
+        # for some items the syntax is a bit different in requirements files so better to do it this way
         with tempfile.TemporaryDirectory() as tmp_dir:
             requirements_file = path.join(tmp_dir, "requirements.txt")
-            with open(requirements_file, "wb") as f:
-                f.write(b"\n".join(requirements_to_uninstall))
+            with open(requirements_file, "w") as f:
+                f.write("\n".join(requirements_to_remove))
             subprocess.run(
                 [
                     venv.spec.pip_path().absolute(),
@@ -234,23 +272,24 @@ def ensure_digest_installed(venv: Venv, target_digest: bytes) -> None:
                 check=True,
             )
 
-    time_since_last_attempt = time.time() - venv.state.when_last_update_attempt
-    if time_since_last_attempt < _MIN_TIME_BETWEEN_ATTEMPTS:
-        time.sleep(_MIN_TIME_BETWEEN_ATTEMPTS - time_since_last_attempt)
-    venv.state.when_last_update_attempt = time.time()
-
     log.info("Installing new requirements...")
-    subprocess.run(
-        [
-            venv.spec.pip_path().absolute(),
-            "install",
-            "-r",
-            venv.spec.requirements_file,
-        ],
-        check=True,
-    )
-    # Technically the requirements_to_install might be out of date at this point.
-    # However this will at worst result in another automatic no-op update later.
+    if requirements_to_install:
+        with tempfile.TemporaryDirectory() as tmp_dir:
+            requirements_file = path.join(tmp_dir, "requirements.txt")
+            with open(requirements_file, "w") as f:
+                f.write("\n".join(requirements_to_install))
+            subprocess.run(
+                [
+                    venv.spec.pip_path().absolute(),
+                    "install",
+                    "-r",
+                    requirements_file,
+                ],
+                check=True,
+            )
+
+    # Technically we might have just installed something else than this digest
+    # In that case the update will be triggered again, but will be mostly noop
     venv.state.installed_digest = target_digest
     venv.state.last_updated_timestamp = time.time()
 
